#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `geohash` package."""

import pytest
from geohash import geohash

geohash_map = (
    ((35.689487, 139.691706), "xn774c"),
    ((35.179554, 129.075642), "wy7b1h"),
    ((48.856614, 2.352222), "u09tvw"),
)


@pytest.mark.parametrize("coordinates, expected", geohash_map)
def test_geohash_encode(coordinates, expected):
    lat, long = coordinates
    # precision >= 0 then precision = 12
    assert geohash.geohash_encode(lat, long, -1) == expected


@pytest.mark.parametrize("expected, hash_code", geohash_map)
def test_geohash_decode(hash_code, expected):
    location = geohash.geohash_decode(hash_code)
    assert all(
        [
            location.longitude,
            location.latitude,
            location.north,
            location.east,
            location.south,
            location.west,
            location.dimension,
        ]
    )



from pathlib import Path

from cffi import FFI

ffi = FFI()
ffi.cdef(
    """
    // Metric in meters
    typedef struct GeoBoxDimensionStruct {
        
        double height;
        double width;
    
    } GeoBoxDimension;
    
    typedef struct GeoCoordStruct {
        
        double latitude;
        double longitude;
        
        double north;
        double east;
        double south;
        double west;
    
        GeoBoxDimension dimension;
        
    } GeoCoord;


    char* geohash_encode(double lat, double lng, int precision);
    GeoCoord geohash_decode(char* hash);
"""
)
_geohash_lib_path = Path(__file__).parent / "bin/linux/geohash.so"
geohash_lib = ffi.dlopen(str(_geohash_lib_path.absolute()))
ffi.set_source("_geohash", None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['cffi>=1.12.3']

setup_requirements = ['pytest-runner', 'cffi']

test_requirements = ['pytest', ]

setup(
    author="Aldrin Navarro",
    author_email='aldrinnavarro16@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    description="Encode/decode Geohashes http://geohash.org",
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='geohash',
    name='geohashcx',
    packages=find_packages(include=['geohash']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/aldnav/geohash',
    version='0.1.1',
    zip_safe=False,
    cffi_modules=["geohash/geohash_build.py:ffi"],  # @TODO: cffi_modules here
)



# -*- coding: utf-8 -*-

"""Main module."""
from .geohash_build import ffi
from .geohash_build import geohash_lib as lib


def geohash_encode(lat: float, long: float, precision: int):
    return ffi.string(lib.geohash_encode(lat, long, precision)).decode()


def geohash_decode(hash_code: str):
    return lib.geohash_decode(hash_code.encode())
